<?php

namespace QUI\ERP\Products\Search;

use QUI;
use QUI\ERP\Products\Handler\Fields;
use QUI\Package\Package;

use function explode;
use function implode;

/**
 * Class Events
 *
 * Main event handler for quiqqer/productsearch
 */
class Events
{
    /**
     * quiqqer/core: onPackageSetup
     *
     * @param Package $Package
     * @return void
     */
    public static function onPackageSetup(Package $Package): void
    {
        if ($Package->getName() !== 'quiqqer/productsearch') {
            return;
        }

        self::patchProductSearchSettings();
        self::patchProductSearchSiteType();
    }

    /**
     * Set default search settings if none are set
     *
     * @return void
     * @throws QUI\Exception
     */
    protected static function setDefaultSearchSettings(): void
    {
        try {
            $Conf = QUI::getPackage('quiqqer/productsearch')->getConfig();
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);

            return;
        }

        $search = $Conf->getSection('search');

        // Backend search fields
        if (empty($search['backend'])) {
            $search['backend'] = implode(',', [
                Fields::FIELD_PRODUCT_NO,
                Fields::FIELD_TITLE
            ]);
        }

        if (empty($search['freetext'])) {
            $search['freetext'] = implode(',', [
                Fields::FIELD_PRODUCT_NO,
                Fields::FIELD_TITLE,
                Fields::FIELD_SHORT_DESC,
                Fields::FIELD_KEYWORDS
            ]);
        }

        $Conf->setSection('search', $search);
        $Conf->save();
    }

    /**
     * Takes the "search" config section from quiqqer/products
     * and writes them to the same config section of quiqqer/productsearch
     *
     */
    protected static function patchProductSearchSettings(): void
    {
        try {
            $ProductsConf = QUI::getPackage('quiqqer/products')->getConfig();
            $ProductSearchConf = QUI::getPackage('quiqqer/productsearch')->getConfig();

            $searchConfig = $ProductsConf->getSection('search');

            if (!empty($searchConfig)) {
                if (!empty($searchConfig['backend'])) {
                    $backend = explode(',', $searchConfig['backend']);
                    $searchConfig['backend'] = implode(',', array_unique($backend));
                }

                if (!empty($searchConfig['frontend'])) {
                    $backend = explode(',', $searchConfig['frontend']);
                    $searchConfig['frontend'] = implode(',', array_unique($backend));
                }

                if (!empty($searchConfig['freetext'])) {
                    $backend = explode(',', $searchConfig['freetext']);
                    $searchConfig['freetext'] = implode(',', array_unique($backend));
                }

                $ProductSearchConf->setSection('search', $searchConfig);
                $ProductSearchConf->save();

                $ProductsConf->del('search');
                $ProductsConf->save();
            } else {
                self::setDefaultSearchSettings();
            }
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
        }
    }

    /**
     * Changes all sites of type "quiqqer/products:type/search"
     * to "quiqqer/productsearch:types/search"
     *
     * @return void
     */
    protected static function patchProductSearchSiteType(): void
    {
        try {
            $Projects = QUI::getProjectManager();
            $projects = $Projects->getProjects(true);

            /** @var QUI\Projects\Project $Project */
            foreach ($projects as $Project) {
                $langs = $Project->getLanguages();

                foreach ($langs as $lang) {
                    try {
                        $EditProject = $Projects->getProject($Project->getName(), $lang);

                        QUI::getDataBase()->update(
                            $EditProject->table(),
                            [
                                'type' => FrontendSearch::SITETYPE_SEARCH
                            ],
                            [
                                'type' => 'quiqqer/products:types/search'
                            ]
                        );
                    } catch (\Exception $Exception) {
                        QUI\System\Log::writeException($Exception);
                    }
                }
            }
        } catch (\Exception $Exception) {
            QUI\System\Log::writeException($Exception);
        }
    }
}
